@@ -1,11 +1,33 @@
 #!/usr/bin/env python
 #
+#
+# Licensed to the Apache Software Foundation (ASF) under one
+# or more contributor license agreements.  See the NOTICE file
+# distributed with this work for additional information
+# regarding copyright ownership.  The ASF licenses this file
+# to you under the Apache License, Version 2.0 (the
+# "License"); you may not use this file except in compliance
+# with the License.  You may obtain a copy of the License at
+#
+#   http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing,
+# software distributed under the License is distributed on an
+# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+# KIND, either express or implied.  See the License for the
+# specific language governing permissions and limitations
+# under the License.
+#
+#
+#
 # gen-make.py -- generate makefiles for building Subversion
 #
 
 
 import os
+import traceback
 import sys
+
 import getopt
 try:
   my_getopt = getopt.gnu_getopt
@@ -43,6 +65,9 @@ def main(fname, gentype, verfname=None,
     generator.compute_hdr_deps()
 
   generator.write()
+  generator.write_sqlite_headers()
+  generator.write_errno_table()
+  generator.write_config_keys()
 
   if ('--debug', '') in other_options:
     for dep_type, target_dict in generator.graph.deps.items():
@@ -55,7 +80,7 @@ def main(fname, gentype, verfname=None,
     gen_keys = sorted(generator.__dict__.keys())
     for name in gen_keys:
       value = generator.__dict__[name]
-      if type(value) == type([]):
+      if isinstance(value, list):
         print(name + ": ")
         for i in value:
           print("  " + _objinfo(i))
@@ -63,7 +88,7 @@ def main(fname, gentype, verfname=None,
 
 
 def _objinfo(o):
-  if type(o) == type(''):
+  if isinstance(o, str):
     return repr(o)
   else:
     t = o.__class__.__name__
@@ -72,8 +97,10 @@ def _objinfo(o):
     return "%s: %s %s" % (t,n,f)
 
 
-def _usage_exit():
-  "print usage, exit the script"
+def _usage_exit(err=None):
+  "print ERR (if any), print usage, then exit the script"
+  if err:
+    print("ERROR: %s\n" % (err))
   print("USAGE:  gen-make.py [options...] [conf-file]")
   print("  -s        skip dependency generation")
   print("  --debug   print lots of stuff only developers care about")
@@ -119,12 +146,6 @@ def _usage_exit():
   print("           look for Berkeley DB headers and libs in")
   print("           DIR")
   print("")
-  print("  --with-neon=DIR")
-  print("           the Neon sources are in DIR")
-  print("")
-  print("  --without-neon")
-  print("           Don't build Neon sources (if present)")
-  print("")
   print("  --with-serf=DIR")
   print("           the Serf sources are in DIR")
   print("")
@@ -139,13 +160,16 @@ def _usage_exit():
   print("           implies --enable-nls")
   print("")
   print("  --with-openssl=DIR")
-  print("           tell neon to look for OpenSSL headers")
+  print("           tell serf to look for OpenSSL headers")
   print("           and libs in DIR")
   print("")
   print("  --with-zlib=DIR")
-  print("           tell neon to look for ZLib headers and")
+  print("           tell Subversion to look for ZLib headers and")
   print("           libs in DIR")
   print("")
+  print("  --with-jdk=DIR")
+  print("           look for the java development kit here")
+  print("")
   print("  --with-junit=DIR")
   print("           look for the junit jar here")
   print("           junit is for testing the java bindings")
@@ -172,21 +196,28 @@ def _usage_exit():
   print("  --enable-nls")
   print("           add support for gettext localization")
   print("")
-  print("  --enable-bdb-in-apr-util")
-  print("           configure APR-Util to use Berkeley DB")
-  print("")
-  print("  --enable-ml")
-  print("           enable use of ML assembler with zlib")
-  print("")
   print("  --disable-shared")
   print("           only build static libraries")
   print("")
+  print("  --with-static-apr")
+  print("           Use static apr and apr-util")
+  print("")
+  print("  --with-static-openssl")
+  print("           Use static openssl")
+  print("")
   print("  --vsnet-version=VER")
-  print("           generate for VS.NET version VER (2002, 2003, 2005 or 2008)")
+  print("           generate for VS.NET version VER (2002, 2003, 2005, 2008,")
+  print("           2010, 2012, 2013 or 2015)")
+  print("           [only valid in combination with '-t vcproj']")
+  print("")
+  print(" -D NAME[=value]")
+  print("           define NAME macro during compilation")
   print("           [only valid in combination with '-t vcproj']")
   print("")
   print("  --with-apr_memcache=DIR")
   print("           the apr_memcache sources are in DIR")
+  print("  --disable-gmock")
+  print("           do not use Googlemock")
   sys.exit(1)
 
 
@@ -195,16 +226,17 @@ class Options:
     self.list = []
     self.dict = {}
 
-  def add(self, opt, val):
+  def add(self, opt, val, overwrite=True):
     if opt in self.dict:
-      self.list[self.dict[opt]] = (opt, val)
+      if overwrite:
+        self.list[self.dict[opt]] = (opt, val)
     else:
       self.dict[opt] = len(self.list)
       self.list.append((opt, val))
 
 if __name__ == '__main__':
   try:
-    opts, args = my_getopt(sys.argv[1:], 'st:',
+    opts, args = my_getopt(sys.argv[1:], 'st:D:',
                            ['debug',
                             'release',
                             'reload',
@@ -213,32 +245,34 @@ if __name__ == '__main__':
                             'with-apr-util=',
                             'with-apr-iconv=',
                             'with-berkeley-db=',
-                            'with-neon=',
-                            'without-neon',
                             'with-serf=',
                             'with-httpd=',
                             'with-libintl=',
                             'with-openssl=',
                             'with-zlib=',
+                            'with-jdk=',
                             'with-junit=',
                             'with-swig=',
                             'with-sqlite=',
                             'with-sasl=',
                             'with-apr_memcache=',
+                            'with-static-apr',
+                            'with-static-openssl',
                             'enable-pool-debug',
                             'enable-purify',
                             'enable-quantify',
                             'enable-nls',
-                            'enable-bdb-in-apr-util',
-                            'enable-ml',
                             'disable-shared',
                             'installed-libs=',
                             'vsnet-version=',
+                            'disable-gmock',
                             ])
     if len(args) > 1:
-      _usage_exit()
+      _usage_exit("Too many arguments")
   except getopt.GetoptError:
-    _usage_exit()
+    typ, val, tb = sys.exc_info()
+    msg = ''.join(traceback.format_exception_only(typ, val))
+    _usage_exit(msg)
 
   conf = 'build.conf'
   skip = 0
@@ -258,6 +292,9 @@ if __name__ == '__main__':
         if opt != '--debug':
           rest.add(opt, val)
       del prev_conf
+    elif opt == '--with-neon' or opt == '--without-neon':
+      # Provide a warning that we ignored these arguments
+      print("Ignoring no longer supported argument '%s'" % opt)
     else:
       rest.add(opt, val)
 
@@ -269,9 +306,12 @@ if __name__ == '__main__':
       gentype = val
     else:
       if opt == '--with-httpd':
-        rest.add('--with-apr', os.path.join(val, 'srclib', 'apr'))
-        rest.add('--with-apr-util', os.path.join(val, 'srclib', 'apr-util'))
-        rest.add('--with-apr-iconv', os.path.join(val, 'srclib', 'apr-iconv'))
+        rest.add('--with-apr', os.path.join(val, 'srclib', 'apr'),
+                 overwrite=False)
+        rest.add('--with-apr-util', os.path.join(val, 'srclib', 'apr-util'),
+                 overwrite=False)
+        rest.add('--with-apr-iconv', os.path.join(val, 'srclib', 'apr-iconv'),
+                 overwrite=False)
 
   # Remember all options so that --reload and other scripts can use them
   opt_conf = open('gen-make.opts', 'w')
@@ -281,7 +321,7 @@ if __name__ == '__main__':
   opt_conf.close()
 
   if gentype not in gen_modules.keys():
-    _usage_exit()
+    _usage_exit("Unknown module type '%s'" % (gentype))
 
   main(conf, gentype, skip_depends=skip, other_options=rest.list)
 
