@@ -2,17 +2,22 @@
  * hash.c :  dumping and reading hash tables to/from files.
  *
  * ====================================================================
- * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
+ *    Licensed to the Apache Software Foundation (ASF) under one
+ *    or more contributor license agreements.  See the NOTICE file
+ *    distributed with this work for additional information
+ *    regarding copyright ownership.  The ASF licenses this file
+ *    to you under the Apache License, Version 2.0 (the
+ *    "License"); you may not use this file except in compliance
+ *    with the License.  You may obtain a copy of the License at
  *
- * This software is licensed as described in the file COPYING, which
- * you should have received as part of this distribution.  The terms
- * are also available at http://subversion.tigris.org/license-1.html.
- * If newer versions of this license are posted there, you may use a
- * newer version instead, at your option.
+ *      http://www.apache.org/licenses/LICENSE-2.0
  *
- * This software consists of voluntary contributions made by many
- * individuals.  For exact contribution history, see the revision
- * history and logs, available at http://subversion.tigris.org/.
+ *    Unless required by applicable law or agreed to in writing,
+ *    software distributed under the License is distributed on an
+ *    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ *    KIND, either express or implied.  See the License for the
+ *    specific language governing permissions and limitations
+ *    under the License.
  * ====================================================================
  */
 
@@ -20,10 +25,12 @@
 
 #include <stdlib.h>
 #include <limits.h>
+
 #include <apr_version.h>
 #include <apr_pools.h>
 #include <apr_hash.h>
 #include <apr_file_io.h>
+
 #include "svn_types.h"
 #include "svn_string.h"
 #include "svn_error.h"
@@ -31,7 +38,13 @@
 #include "svn_sorts.h"
 #include "svn_io.h"
 #include "svn_pools.h"
+
 #include "private/svn_dep_compat.h"
+#include "private/svn_sorts_private.h"
+#include "private/svn_subr_private.h"
+
+#include "svn_private_config.h"
+
 
 
 
@@ -77,101 +90,156 @@
 /*** Dumping and loading hash files. */
 
 /* Implements svn_hash_read2 and svn_hash_read_incremental. */
+svn_error_t *
+svn_hash__read_entry(svn_hash__entry_t *entry,
+                     svn_stream_t *stream,
+                     const char *terminator,
+                     svn_boolean_t incremental,
+                     apr_pool_t *pool)
+{
+  svn_stringbuf_t *buf;
+  svn_boolean_t eof;
+  apr_size_t len;
+  char c;
+
+  svn_error_t *err;
+  apr_uint64_t ui64;
+
+  /* Read a key length line.  Might be END, though. */
+  SVN_ERR(svn_stream_readline(stream, &buf, "\n", &eof, pool));
+
+  /* Check for the end of the hash. */
+  if ((!terminator && eof && buf->len == 0)
+      || (terminator && (strcmp(buf->data, terminator) == 0)))
+  {
+    entry->key = NULL;
+    entry->keylen = 0;
+    entry->val = NULL;
+    entry->vallen = 0;
+
+    return SVN_NO_ERROR;
+  }
+
+  /* Check for unexpected end of stream */
+  if (eof)
+    return svn_error_create(SVN_ERR_MALFORMED_FILE, NULL,
+                            _("Serialized hash missing terminator"));
+
+  if ((buf->len >= 3) && (buf->data[0] == 'K') && (buf->data[1] == ' '))
+    {
+      /* Get the length of the key */
+      err = svn_cstring_strtoui64(&ui64, buf->data + 2,
+                                  0, APR_SIZE_MAX, 10);
+      if (err)
+        return svn_error_create(SVN_ERR_MALFORMED_FILE, err,
+                                _("Serialized hash malformed key length"));
+      entry->keylen = (apr_size_t)ui64;
+
+      /* Now read that much into a buffer. */
+      entry->key = apr_palloc(pool, entry->keylen + 1);
+      SVN_ERR(svn_stream_read_full(stream, entry->key, &entry->keylen));
+      entry->key[entry->keylen] = '\0';
+
+      /* Suck up extra newline after key data */
+      len = 1;
+      SVN_ERR(svn_stream_read_full(stream, &c, &len));
+      if (c != '\n')
+        return svn_error_create(SVN_ERR_MALFORMED_FILE, NULL,
+                                _("Serialized hash malformed key data"));
+
+      /* Read a val length line */
+      SVN_ERR(svn_stream_readline(stream, &buf, "\n", &eof, pool));
+
+      if ((buf->data[0] == 'V') && (buf->data[1] == ' '))
+        {
+          /* Get the length of the val */
+          err = svn_cstring_strtoui64(&ui64, buf->data + 2,
+                                      0, APR_SIZE_MAX, 10);
+          if (err)
+            return svn_error_create(SVN_ERR_MALFORMED_FILE, err,
+                                    _("Serialized hash malformed value length"));
+          entry->vallen = (apr_size_t)ui64;
+
+          entry->val = apr_palloc(pool, entry->vallen + 1);
+          SVN_ERR(svn_stream_read_full(stream, entry->val, &entry->vallen));
+          entry->val[entry->vallen] = '\0';
+
+          /* Suck up extra newline after val data */
+          len = 1;
+          SVN_ERR(svn_stream_read_full(stream, &c, &len));
+          if (c != '\n')
+            return svn_error_create(SVN_ERR_MALFORMED_FILE, NULL,
+                                    _("Serialized hash malformed value data"));
+        }
+      else
+        return svn_error_create(SVN_ERR_MALFORMED_FILE, NULL,
+                                _("Serialized hash malformed"));
+    }
+  else if (incremental && (buf->len >= 3)
+           && (buf->data[0] == 'D') && (buf->data[1] == ' '))
+    {
+      /* Get the length of the key */
+      err = svn_cstring_strtoui64(&ui64, buf->data + 2,
+                                  0, APR_SIZE_MAX, 10);
+      if (err)
+        return svn_error_create(SVN_ERR_MALFORMED_FILE, err,
+                                _("Serialized hash malformed key length"));
+      entry->keylen = (apr_size_t)ui64;
+
+      /* Now read that much into a buffer. */
+      entry->key = apr_palloc(pool, entry->keylen + 1);
+      SVN_ERR(svn_stream_read_full(stream, entry->key, &entry->keylen));
+      entry->key[entry->keylen] = '\0';
+
+      /* Suck up extra newline after key data */
+      len = 1;
+      SVN_ERR(svn_stream_read_full(stream, &c, &len));
+      if (c != '\n')
+        return svn_error_create(SVN_ERR_MALFORMED_FILE, NULL,
+                                _("Serialized hash malformed key data"));
+
+      /* Remove this hash entry. */
+      entry->vallen = 0;
+      entry->val = NULL;
+    }
+  else
+    {
+      return svn_error_create(SVN_ERR_MALFORMED_FILE, NULL,
+                              _("Serialized hash malformed"));
+    }
+
+  return SVN_NO_ERROR;
+}
+
 static svn_error_t *
 hash_read(apr_hash_t *hash, svn_stream_t *stream, const char *terminator,
           svn_boolean_t incremental, apr_pool_t *pool)
 {
-  svn_stringbuf_t *buf;
-  svn_boolean_t eof;
-  apr_size_t len, keylen, vallen;
-  char c, *end, *keybuf, *valbuf;
   apr_pool_t *iterpool = svn_pool_create(pool);
 
   while (1)
     {
+      svn_hash__entry_t entry;
+
       svn_pool_clear(iterpool);
+      SVN_ERR(svn_hash__read_entry(&entry, stream, terminator,
+                                   incremental, iterpool));
 
-      /* Read a key length line.  Might be END, though. */
-      SVN_ERR(svn_stream_readline(stream, &buf, "\n", &eof, iterpool));
-
-      /* Check for the end of the hash. */
-      if ((!terminator && eof && buf->len == 0)
-          || (terminator && (strcmp(buf->data, terminator) == 0)))
+      /* end of hash? */
+      if (entry.key == NULL)
         break;
 
-      /* Check for unexpected end of stream */
-      if (eof)
-        return svn_error_create(SVN_ERR_MALFORMED_FILE, NULL, NULL);
-
-      if ((buf->len >= 3) && (buf->data[0] == 'K') && (buf->data[1] == ' '))
+      if (entry.val)
         {
-          /* Get the length of the key */
-          keylen = (size_t) strtoul(buf->data + 2, &end, 10);
-          if (keylen == (size_t) ULONG_MAX || *end != '\0')
-            return svn_error_create(SVN_ERR_MALFORMED_FILE, NULL, NULL);
-
-          /* Now read that much into a buffer. */
-          keybuf = apr_palloc(pool, keylen + 1);
-          SVN_ERR(svn_stream_read(stream, keybuf, &keylen));
-          keybuf[keylen] = '\0';
-
-          /* Suck up extra newline after key data */
-          len = 1;
-          SVN_ERR(svn_stream_read(stream, &c, &len));
-          if (c != '\n')
-            return svn_error_create(SVN_ERR_MALFORMED_FILE, NULL, NULL);
-
-          /* Read a val length line */
-          SVN_ERR(svn_stream_readline(stream, &buf, "\n", &eof, iterpool));
-
-          if ((buf->data[0] == 'V') && (buf->data[1] == ' '))
-            {
-              vallen = (size_t) strtoul(buf->data + 2, &end, 10);
-              if (vallen == (size_t) ULONG_MAX || *end != '\0')
-                return svn_error_create(SVN_ERR_MALFORMED_FILE, NULL, NULL);
-
-              valbuf = apr_palloc(iterpool, vallen + 1);
-              SVN_ERR(svn_stream_read(stream, valbuf, &vallen));
-              valbuf[vallen] = '\0';
-
-              /* Suck up extra newline after val data */
-              len = 1;
-              SVN_ERR(svn_stream_read(stream, &c, &len));
-              if (c != '\n')
-                return svn_error_create(SVN_ERR_MALFORMED_FILE, NULL, NULL);
-
-              /* Add a new hash entry. */
-              apr_hash_set(hash, keybuf, keylen,
-                           svn_string_ncreate(valbuf, vallen, pool));
-            }
-          else
-            return svn_error_create(SVN_ERR_MALFORMED_FILE, NULL, NULL);
-        }
-      else if (incremental && (buf->len >= 3)
-               && (buf->data[0] == 'D') && (buf->data[1] == ' '))
-        {
-          /* Get the length of the key */
-          keylen = (size_t) strtoul(buf->data + 2, &end, 10);
-          if (keylen == (size_t) ULONG_MAX || *end != '\0')
-            return svn_error_create(SVN_ERR_MALFORMED_FILE, NULL, NULL);
-
-          /* Now read that much into a buffer. */
-          keybuf = apr_palloc(iterpool, keylen + 1);
-          SVN_ERR(svn_stream_read(stream, keybuf, &keylen));
-          keybuf[keylen] = '\0';
-
-          /* Suck up extra newline after key data */
-          len = 1;
-          SVN_ERR(svn_stream_read(stream, &c, &len));
-          if (c != '\n')
-            return svn_error_create(SVN_ERR_MALFORMED_FILE, NULL, NULL);
-
-          /* Remove this hash entry. */
-          apr_hash_set(hash, keybuf, keylen, NULL);
+          /* Add a new hash entry. */
+          apr_hash_set(hash, apr_pstrmemdup(pool, entry.key, entry.keylen),
+                       entry.keylen,
+                       svn_string_ncreate(entry.val, entry.vallen, pool));
         }
       else
         {
-          return svn_error_create(SVN_ERR_MALFORMED_FILE, NULL, NULL);
+          /* Remove this hash entry. */
+          apr_hash_set(hash, entry.key, entry.keylen, NULL);
         }
     }
 
@@ -209,15 +277,20 @@ hash_write(apr_hash_t *hash, apr_hash_t *oldhash, svn_stream_t *stream,
             continue;
         }
 
+      if (item->klen < 0)
+        return svn_error_create(SVN_ERR_MALFORMED_FILE, NULL,
+                                _("Cannot serialize negative length"));
+
       /* Write it out. */
       SVN_ERR(svn_stream_printf(stream, subpool,
-                                "K %" APR_SSIZE_T_FMT "\n%s\n"
+                                "K %" APR_SIZE_T_FMT "\n%s\n"
                                 "V %" APR_SIZE_T_FMT "\n",
-                                item->klen, (const char *) item->key,
+                                (apr_size_t) item->klen,
+                                (const char *) item->key,
                                 valstr->len));
       len = valstr->len;
       SVN_ERR(svn_stream_write(stream, valstr->data, &len));
-      SVN_ERR(svn_stream_printf(stream, subpool, "\n"));
+      SVN_ERR(svn_stream_puts(stream, "\n"));
     }
 
   if (oldhash)
@@ -339,13 +412,19 @@ svn_hash_read(apr_hash_t *hash,
         }
       else if ((buf[0] == 'K') && (buf[1] == ' '))
         {
+          size_t keylen;
+          int parsed_len;
+          void *keybuf;
+
           /* Get the length of the key */
-          size_t keylen = (size_t) atoi(buf + 2);
+          SVN_ERR(svn_cstring_atoi(&parsed_len, buf + 2));
+          keylen = parsed_len;
 
           /* Now read that much into a buffer, + 1 byte for null terminator */
-          void *keybuf = apr_palloc(pool, keylen + 1);
-          SVN_ERR(svn_io_file_read_full(srcfile,
-                                        keybuf, keylen, &num_read, pool));
+          keybuf = apr_palloc(pool, keylen + 1);
+          SVN_ERR(svn_io_file_read_full2(srcfile,
+                                         keybuf, keylen,
+                                         &num_read, NULL, pool));
           ((char *) keybuf)[keylen] = '\0';
 
           /* Suck up extra newline after key data */
@@ -360,15 +439,18 @@ svn_hash_read(apr_hash_t *hash,
           if ((buf[0] == 'V') && (buf[1] == ' '))
             {
               svn_string_t *value = apr_palloc(pool, sizeof(*value));
+              apr_size_t vallen;
+              void *valbuf;
 
               /* Get the length of the value */
-              apr_size_t vallen = atoi(buf + 2);
+              SVN_ERR(svn_cstring_atoi(&parsed_len, buf + 2));
+              vallen = parsed_len;
 
               /* Again, 1 extra byte for the null termination. */
-              void *valbuf = apr_palloc(pool, vallen + 1);
-              SVN_ERR(svn_io_file_read_full(srcfile,
-                                            valbuf, vallen,
-                                            &num_read, pool));
+              valbuf = apr_palloc(pool, vallen + 1);
+              SVN_ERR(svn_io_file_read_full2(srcfile,
+                                             valbuf, vallen,
+                                             &num_read, NULL, pool));
               ((char *) valbuf)[vallen] = '\0';
 
               /* Suck up extra newline after val data */
@@ -453,13 +535,7 @@ svn_hash_keys(apr_array_header_t **array,
 
   for (hi = apr_hash_first(pool, hash); hi; hi = apr_hash_next(hi))
     {
-      const void *key;
-      const char *path;
-
-      apr_hash_this(hi, &key, NULL, NULL);
-      path = key;
-
-      APR_ARRAY_PUSH(*array, const char *) = path;
+      APR_ARRAY_PUSH(*array, const char *) = apr_hash_this_key(hi);
     }
 
   return SVN_NO_ERROR;
@@ -472,33 +548,104 @@ svn_hash_from_cstring_keys(apr_hash_t **hash_p,
                            apr_pool_t *pool)
 {
   int i;
-  apr_hash_t *hash = apr_hash_make(pool);
+  apr_hash_t *hash = svn_hash__make(pool);
   for (i = 0; i < keys->nelts; i++)
     {
       const char *key =
         apr_pstrdup(pool, APR_ARRAY_IDX(keys, i, const char *));
-      apr_hash_set(hash, key, APR_HASH_KEY_STRING, key);
+      svn_hash_sets(hash, key, key);
     }
   *hash_p = hash;
   return SVN_NO_ERROR;
 }
 
 
-svn_error_t *
-svn_hash__clear(apr_hash_t *hash)
-{
-#if APR_VERSION_AT_LEAST(1, 3, 0)
-  apr_hash_clear(hash);
-#else
-  apr_hash_index_t *hi;
-  const void *key;
-  apr_ssize_t klen;
+
+/*** Specialized getter APIs ***/
 
-  for (hi = apr_hash_first(NULL, hash); hi; hi = apr_hash_next(hi))
+const char *
+svn_hash__get_cstring(apr_hash_t *hash,
+                      const char *key,
+                      const char *default_value)
+{
+  if (hash)
     {
-      apr_hash_this(hi, &key, &klen, NULL);
-      apr_hash_set(hash, key, klen, NULL);
+      const char *value = svn_hash_gets(hash, key);
+      return value ? value : default_value;
     }
+
+  return default_value;
+}
+
+
+svn_boolean_t
+svn_hash__get_bool(apr_hash_t *hash, const char *key,
+                   svn_boolean_t default_value)
+{
+  const char *tmp_value = svn_hash__get_cstring(hash, key, NULL);
+  svn_tristate_t value = svn_tristate__from_word(tmp_value);
+
+  if (value == svn_tristate_true)
+    return TRUE;
+  else if (value == svn_tristate_false)
+    return FALSE;
+
+  return default_value;
+}
+
+
+
+/*** Optimized hash function ***/
+
+/* apr_hashfunc_t optimized for the key that we use in SVN: paths and
+ * property names.  Its primary goal is speed for keys of known length.
+ *
+ * Since strings tend to spawn large value spaces (usually differ in many
+ * bits with differences spanning a larger section of the key), we can be
+ * quite sloppy extracting a hash value.  The more keys there are in a
+ * hash container, the more bits of the value returned by this function
+ * will be used.  For a small number of string keys, choosing bits from any
+ * any fix location close to the tail of those keys would usually be good
+ * enough to prevent high collision rates.
+ */
+static unsigned int
+hashfunc_compatible(const char *char_key, apr_ssize_t *klen)
+{
+    unsigned int hash = 0;
+    const unsigned char *key = (const unsigned char *)char_key;
+    const unsigned char *p;
+    apr_ssize_t i;
+
+    if (*klen == APR_HASH_KEY_STRING)
+      *klen = strlen(char_key);
+
+#if SVN_UNALIGNED_ACCESS_IS_OK
+    for (p = key, i = *klen; i >= 4; i-=4, p+=4)
+      {
+        apr_uint32_t chunk = *(const apr_uint32_t *)p;
+
+        /* the ">> 17" part gives upper bits in the chunk a chance to make
+           some impact as well */
+        hash = hash * 33 * 33 * 33 * 33 + chunk + (chunk >> 17);
+      }
+#else
+    for (p = key, i = *klen; i >= 4; i-=4, p+=4)
+      {
+        hash = hash * 33 * 33 * 33 * 33
+              + p[0] * 33 * 33 * 33
+              + p[1] * 33 * 33
+              + p[2] * 33
+              + p[3];
+      }
 #endif
-  return SVN_NO_ERROR;
+    for (; i; i--, p++)
+        hash = hash * 33 + *p;
+
+    return hash;
+}
+
+apr_hash_t *
+svn_hash__make(apr_pool_t *pool)
+{
+  return apr_hash_make_custom(pool, hashfunc_compatible);
 }
