@@ -1,18 +1,23 @@
 /*
- * replay.c :  routines for replaying revisions
+ * replay.c: mod_dav_svn REPORT handler for replaying revisions
  *
  * ====================================================================
- * Copyright (c) 2005, 2008 CollabNet.  All rights reserved.
+ *    Licensed to the Apache Software Foundation (ASF) under one
+ *    or more contributor license agreements.  See the NOTICE file
+ *    distributed with this work for additional information
+ *    regarding copyright ownership.  The ASF licenses this file
+ *    to you under the Apache License, Version 2.0 (the
+ *    "License"); you may not use this file except in compliance
+ *    with the License.  You may obtain a copy of the License at
  *
- * This software is licensed as described in the file COPYING, which
- * you should have received as part of this distribution.  The terms
- * are also available at http://subversion.tigris.org/license-1.html.
- * If newer versions of this license are posted there, you may use a
- * newer version instead, at your option.
+ *      http://www.apache.org/licenses/LICENSE-2.0
  *
- * This software consists of voluntary contributions made by many
- * individuals.  For exact contribution history, see the revision
- * history and logs, available at http://subversion.tigris.org/.
+ *    Unless required by applicable law or agreed to in writing,
+ *    software distributed under the License is distributed on an
+ *    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ *    KIND, either express or implied.  See the License for the
+ *    specific language governing permissions and limitations
+ *    under the License.
  * ====================================================================
  */
 
@@ -37,11 +42,12 @@
 #include "../dav_svn.h"
 
 
-typedef struct {
+typedef struct edit_baton_t {
   apr_bucket_brigade *bb;
   ap_filter_t *output;
   svn_boolean_t started;
   svn_boolean_t sending_textdelta;
+  int compression_level;
 } edit_baton_t;
 
 
@@ -53,12 +59,10 @@ maybe_start_report(edit_baton_t *eb)
 {
   if (! eb->started)
     {
-      SVN_ERR(dav_svn__send_xml
-                (eb->bb, eb->output,
-                 DAV_XML_HEADER DEBUG_CR
-                 "<S:editor-report xmlns:S=\"" SVN_XML_NAMESPACE "\">"
-                 DEBUG_CR));
-
+      SVN_ERR(dav_svn__brigade_puts(eb->bb, eb->output,
+                                    DAV_XML_HEADER DEBUG_CR
+                                    "<S:editor-report xmlns:S=\""
+                                    SVN_XML_NAMESPACE "\">" DEBUG_CR));
       eb->started = TRUE;
     }
 
@@ -68,8 +72,8 @@ maybe_start_report(edit_baton_t *eb)
 static svn_error_t *
 end_report(edit_baton_t *eb)
 {
-  SVN_ERR(dav_svn__send_xml(eb->bb, eb->output,
-                            "</S:editor-report>" DEBUG_CR));
+  SVN_ERR(dav_svn__brigade_puts(eb->bb, eb->output,
+                                "</S:editor-report>" DEBUG_CR));
 
   return SVN_NO_ERROR;
 }
@@ -80,8 +84,8 @@ maybe_close_textdelta(edit_baton_t *eb)
 {
   if (eb->sending_textdelta)
     {
-      SVN_ERR(dav_svn__send_xml(eb->bb, eb->output,
-                                "</S:apply-textdelta>" DEBUG_CR));
+      SVN_ERR(dav_svn__brigade_puts(eb->bb, eb->output,
+                                    "</S:apply-textdelta>" DEBUG_CR));
       eb->sending_textdelta = FALSE;
     }
 
@@ -104,17 +108,19 @@ add_file_or_directory(const char *file_or_directory,
 
   SVN_ERR(maybe_close_textdelta(eb));
 
-  *added_baton = (void *)eb;
+  *added_baton = eb;
 
   if (! copyfrom_path)
-    SVN_ERR(dav_svn__send_xml(eb->bb, eb->output,
-                              "<S:add-%s name=\"%s\"/>" DEBUG_CR,
-                              file_or_directory, qname));
+    SVN_ERR(dav_svn__brigade_printf(eb->bb, eb->output,
+                                    "<S:add-%s name=\"%s\"/>" DEBUG_CR,
+                                    file_or_directory, qname));
   else
-    SVN_ERR(dav_svn__send_xml(eb->bb, eb->output,
-                              "<S:add-%s name=\"%s\" copyfrom-path=\"%s\" "
-                                        "copyfrom-rev=\"%ld\"/>" DEBUG_CR,
-                              file_or_directory, qname, qcopy, copyfrom_rev));
+    SVN_ERR(dav_svn__brigade_printf(eb->bb, eb->output,
+                                    "<S:add-%s name=\"%s\" "
+                                    "copyfrom-path=\"%s\" "
+                                    "copyfrom-rev=\"%ld\"/>" DEBUG_CR,
+                                    file_or_directory, qname,
+                                    qcopy, copyfrom_rev));
 
   return SVN_NO_ERROR;
 }
@@ -129,10 +135,11 @@ open_file_or_directory(const char *file_or_directory,
 {
   const char *qname = apr_xml_quote_string(pool, path, 1);
   SVN_ERR(maybe_close_textdelta(eb));
-  *opened_baton = (void *)eb;
-  return dav_svn__send_xml(eb->bb, eb->output,
-                           "<S:open-%s name=\"%s\" rev=\"%ld\"/>" DEBUG_CR,
-                           file_or_directory, qname, base_revision);
+  *opened_baton = eb;
+  return dav_svn__brigade_printf(eb->bb, eb->output,
+                                 "<S:open-%s name=\"%s\" rev=\"%ld\"/>"
+                                 DEBUG_CR,
+                                 file_or_directory, qname, base_revision);
 }
 
 
@@ -149,7 +156,6 @@ change_file_or_dir_prop(const char *file_or_dir,
 
   if (value)
     {
-      apr_status_t apr_err;
       const svn_string_t *enc_value =
         svn_base64_encode_string2(value, TRUE, pool);
 
@@ -157,22 +163,21 @@ change_file_or_dir_prop(const char *file_or_dir,
          bug that can be triggered by just the wrong size of a large
          property value.  The bug has been fixed (see
          http://svn.apache.org/viewvc?view=rev&revision=768417), but
-         we need a workaround for the buggy APR versions. */
-      SVN_ERR(dav_svn__send_xml(eb->bb, eb->output,
-                                "<S:change-%s-prop name=\"%s\">",
-                                file_or_dir, qname));
-      if ((apr_err = apr_brigade_write(eb->bb, ap_filter_flush, eb->output,
-                                       enc_value->data, enc_value->len)))
-        return svn_error_create(apr_err, 0, NULL);
-      if (eb->output->c->aborted)
-        return svn_error_create(SVN_ERR_APMOD_CONNECTION_ABORTED, 0, NULL);
-      SVN_ERR(dav_svn__send_xml(eb->bb, eb->output,
-                                "</S:change-%s-prop>" DEBUG_CR,
-                                file_or_dir));
+         we need a workaround for the buggy APR versions, so we write
+         our potentially large block of property data using a
+         different underlying function. */
+      SVN_ERR(dav_svn__brigade_printf(eb->bb, eb->output,
+                                      "<S:change-%s-prop name=\"%s\">",
+                                      file_or_dir, qname));
+      SVN_ERR(dav_svn__brigade_write(eb->bb, eb->output,
+                                     enc_value->data, enc_value->len));
+      SVN_ERR(dav_svn__brigade_printf(eb->bb, eb->output,
+                                      "</S:change-%s-prop>" DEBUG_CR,
+                                      file_or_dir));
     }
   else
     {
-      SVN_ERR(dav_svn__send_xml
+      SVN_ERR(dav_svn__brigade_printf
                 (eb->bb, eb->output,
                  "<S:change-%s-prop name=\"%s\" del=\"true\"/>" DEBUG_CR,
                  file_or_dir, qname));
@@ -192,9 +197,9 @@ set_target_revision(void *edit_baton,
 {
   edit_baton_t *eb = edit_baton;
   SVN_ERR(maybe_start_report(eb));
-  return dav_svn__send_xml(eb->bb, eb->output,
-                           "<S:target-revision rev=\"%ld\"/>" DEBUG_CR,
-                           target_revision);
+  return dav_svn__brigade_printf(eb->bb, eb->output,
+                                 "<S:target-revision rev=\"%ld\"/>" DEBUG_CR,
+                                 target_revision);
 }
 
 
@@ -207,9 +212,9 @@ open_root(void *edit_baton,
   edit_baton_t *eb = edit_baton;
   *root_baton = edit_baton;
   SVN_ERR(maybe_start_report(eb));
-  return dav_svn__send_xml(eb->bb, eb->output,
-                           "<S:open-root rev=\"%ld\"/>" DEBUG_CR,
-                           base_revision);
+  return dav_svn__brigade_printf(eb->bb, eb->output,
+                                 "<S:open-root rev=\"%ld\"/>" DEBUG_CR,
+                                 base_revision);
 }
 
 
@@ -222,9 +227,10 @@ delete_entry(const char *path,
   edit_baton_t *eb = parent_baton;
   const char *qname = apr_xml_quote_string(pool, path, 1);
   SVN_ERR(maybe_close_textdelta(eb));
-  return dav_svn__send_xml(eb->bb, eb->output,
-                           "<S:delete-entry name=\"%s\" rev=\"%ld\"/>" DEBUG_CR,
-                            qname, revision);
+  return dav_svn__brigade_printf(eb->bb, eb->output,
+                                 "<S:delete-entry name=\"%s\" rev=\"%ld\"/>"
+                                 DEBUG_CR,
+                                 qname, revision);
 }
 
 
@@ -307,20 +313,21 @@ apply_textdelta(void *file_baton,
 {
   edit_baton_t *eb = file_baton;
 
-  SVN_ERR(dav_svn__send_xml(eb->bb, eb->output, "<S:apply-textdelta"));
+  SVN_ERR(dav_svn__brigade_puts(eb->bb, eb->output, "<S:apply-textdelta"));
 
   if (base_checksum)
-    SVN_ERR(dav_svn__send_xml(eb->bb, eb->output, " checksum=\"%s\">",
-                              base_checksum));
+    SVN_ERR(dav_svn__brigade_printf(eb->bb, eb->output, " checksum=\"%s\">",
+                                    base_checksum));
   else
-    SVN_ERR(dav_svn__send_xml(eb->bb, eb->output, ">"));
+    SVN_ERR(dav_svn__brigade_puts(eb->bb, eb->output, ">"));
 
-  svn_txdelta_to_svndiff2(handler,
+  svn_txdelta_to_svndiff3(handler,
                           handler_baton,
                           dav_svn__make_base64_output_stream(eb->bb,
                                                              eb->output,
                                                              pool),
                           0,
+                          eb->compression_level,
                           pool);
 
   eb->sending_textdelta = TRUE;
@@ -334,13 +341,14 @@ close_file(void *file_baton, const char *text_checksum, apr_pool_t *pool)
 {
   edit_baton_t *eb = file_baton;
   SVN_ERR(maybe_close_textdelta(eb));
-  SVN_ERR(dav_svn__send_xml(eb->bb, eb->output, "<S:close-file"));
+  SVN_ERR(dav_svn__brigade_puts(eb->bb, eb->output, "<S:close-file"));
 
   if (text_checksum)
-    SVN_ERR(dav_svn__send_xml(eb->bb, eb->output, " checksum=\"%s\"/>" DEBUG_CR,
-                              text_checksum));
+    SVN_ERR(dav_svn__brigade_printf(eb->bb, eb->output,
+                                    " checksum=\"%s\"/>" DEBUG_CR,
+                                    text_checksum));
   else
-    SVN_ERR(dav_svn__send_xml(eb->bb, eb->output, "/>" DEBUG_CR));
+    SVN_ERR(dav_svn__brigade_puts(eb->bb, eb->output, "/>" DEBUG_CR));
 
   return SVN_NO_ERROR;
 }
@@ -350,7 +358,8 @@ static svn_error_t *
 close_directory(void *dir_baton, apr_pool_t *pool)
 {
   edit_baton_t *eb = dir_baton;
-  return dav_svn__send_xml(eb->bb, eb->output, "<S:close-directory/>" DEBUG_CR);
+  return dav_svn__brigade_puts(eb->bb, eb->output,
+                               "<S:close-directory/>" DEBUG_CR);
 }
 
 
@@ -359,6 +368,7 @@ make_editor(const svn_delta_editor_t **editor,
             void **edit_baton,
             apr_bucket_brigade *bb,
             ap_filter_t *output,
+            int compression_level,
             apr_pool_t *pool)
 {
   edit_baton_t *eb = apr_pcalloc(pool, sizeof(*eb));
@@ -368,6 +378,7 @@ make_editor(const svn_delta_editor_t **editor,
   eb->output = output;
   eb->started = FALSE;
   eb->sending_textdelta = FALSE;
+  eb->compression_level = compression_level;
 
   e->set_target_revision = set_target_revision;
   e->open_root = open_root;
@@ -390,13 +401,12 @@ make_editor(const svn_delta_editor_t **editor,
 static dav_error *
 malformed_element_error(const char *tagname, apr_pool_t *pool)
 {
-  return dav_svn__new_error_tag(pool, HTTP_BAD_REQUEST, 0,
+  return dav_svn__new_error_svn(pool, HTTP_BAD_REQUEST, 0,
                                 apr_pstrcat(pool,
                                             "The request's '", tagname,
                                             "' element is malformed; there "
                                             "is a problem with the client.",
-                                            NULL),
-                                SVN_DAV_ERROR_NAMESPACE, SVN_DAV_ERROR_TAG);
+                                            SVN_VA_NULL));
 }
 
 
@@ -407,11 +417,11 @@ dav_svn__replay_report(const dav_resource *resource,
 {
   dav_error *derr = NULL;
   svn_revnum_t low_water_mark = SVN_INVALID_REVNUM;
-  svn_revnum_t rev = SVN_INVALID_REVNUM;
+  svn_revnum_t rev;
   const svn_delta_editor_t *editor;
   svn_boolean_t send_deltas = TRUE;
   dav_svn__authz_read_baton arb;
-  const char *base_dir = resource->info->repos_path;
+  const char *base_dir;
   apr_bucket_brigade *bb;
   apr_xml_elem *child;
   svn_fs_root_t *root;
@@ -419,22 +429,37 @@ dav_svn__replay_report(const dav_resource *resource,
   void *edit_baton;
   int ns;
 
-  /* The request won't have a repos_path if it's for the root. */
-  if (! base_dir)
-    base_dir = "";
+  /* In Subversion 1.8, we allowed this REPORT to be issue against a
+     revision resource.  Doing so means the REV is part of the request
+     URL, and BASE_DIR is embedded in the request body.
+
+     The old-school (and incorrect, see issue #4287 --
+     http://subversion.tigris.org/issues/show_bug.cgi?id=4287) way was
+     to REPORT on the public URL of the BASE_DIR and embed the REV in
+     the report body.
+  */
+  if (resource->baselined
+      && (resource->type == DAV_RESOURCE_TYPE_VERSION))
+    {
+      rev = resource->info->root.rev;
+      base_dir = NULL;
+    }
+  else
+    {
+      rev = SVN_INVALID_REVNUM;
+      base_dir = resource->info->repos_path;
+    }
 
   arb.r = resource->info->r;
   arb.repos = resource->info->repos;
 
   ns = dav_svn__find_ns(doc->namespaces, SVN_XML_NAMESPACE);
   if (ns == -1)
-    return dav_svn__new_error_tag(resource->pool, HTTP_BAD_REQUEST, 0,
+    return dav_svn__new_error_svn(resource->pool, HTTP_BAD_REQUEST, 0,
                                   "The request does not contain the 'svn:' "
                                   "namespace, so it is not going to have an "
                                   "svn:revision element. That element is "
-                                  "required.",
-                                  SVN_DAV_ERROR_NAMESPACE,
-                                  SVN_DAV_ERROR_TAG);
+                                  "required");
 
   for (child = doc->root->first_child; child != NULL; child = child->next)
     {
@@ -444,9 +469,17 @@ dav_svn__replay_report(const dav_resource *resource,
 
           if (strcmp(child->name, "revision") == 0)
             {
+              if (SVN_IS_VALID_REVNUM(rev))
+                {
+                  /* Uh... we already have a revision to use, either
+                     because this tag is non-unique or because the
+                     request was submitted against a revision-bearing
+                     resource URL.  Either way, something's not
+                     right.  */
+                  return malformed_element_error("revision", resource->pool);
+                }
+
               cdata = dav_xml_get_cdata(child, resource->pool, 1);
-              if (! cdata)
-                return malformed_element_error("revision", resource->pool);
               rev = SVN_STR_TO_REV(cdata);
             }
           else if (strcmp(child->name, "low-water-mark") == 0)
@@ -459,25 +492,43 @@ dav_svn__replay_report(const dav_resource *resource,
             }
           else if (strcmp(child->name, "send-deltas") == 0)
             {
+              apr_int64_t parsed_val;
+
               cdata = dav_xml_get_cdata(child, resource->pool, 1);
               if (! cdata)
                 return malformed_element_error("send-deltas", resource->pool);
-              send_deltas = atoi(cdata);
+              err = svn_cstring_strtoi64(&parsed_val, cdata, 0, 1, 10);
+              if (err)
+                {
+                  svn_error_clear(err);
+                  return malformed_element_error("send-deltas", resource->pool);
+                }
+              send_deltas = parsed_val != 0;
+            }
+          else if (strcmp(child->name, "include-path") == 0)
+            {
+              cdata = dav_xml_get_cdata(child, resource->pool, 1);
+              if ((derr = dav_svn__test_canonical(cdata, resource->pool)))
+                return derr;
+
+              /* Force BASE_DIR to be a relative path, not an fspath. */
+              base_dir = svn_relpath_canonicalize(cdata, resource->pool);
             }
         }
     }
 
   if (! SVN_IS_VALID_REVNUM(rev))
-    return dav_svn__new_error_tag
+    return dav_svn__new_error_svn
              (resource->pool, HTTP_BAD_REQUEST, 0,
-              "Request was missing the revision argument.",
-              SVN_DAV_ERROR_NAMESPACE, SVN_DAV_ERROR_TAG);
+              "Request was missing the revision argument");
 
   if (! SVN_IS_VALID_REVNUM(low_water_mark))
-    return dav_svn__new_error_tag
+    return dav_svn__new_error_svn
              (resource->pool, HTTP_BAD_REQUEST, 0,
-              "Request was missing the low-water-mark argument.",
-              SVN_DAV_ERROR_NAMESPACE, SVN_DAV_ERROR_TAG);
+              "Request was missing the low-water-mark argument");
+
+  if (! base_dir)
+    base_dir = "";
 
   bb = apr_brigade_create(resource->pool, output->c->bucket_alloc);
 
@@ -490,7 +541,9 @@ dav_svn__replay_report(const dav_resource *resource,
       goto cleanup;
     }
 
-  make_editor(&editor, &edit_baton, bb, output, resource->pool);
+  make_editor(&editor, &edit_baton, bb, output,
+              dav_svn__get_compression_level(resource->info->r),
+              resource->pool);
 
   if ((err = svn_repos_replay2(root, base_dir, low_water_mark,
                                send_deltas, editor, edit_baton,
