@@ -84,8 +84,7 @@ def export_nonexistent_url(sbox):
   svntest.main.safe_rmtree(sbox.wc_dir)
   export_target = os.path.join(sbox.wc_dir, 'nonexistent')
   nonexistent_url = sbox.repo_url + "/nonexistent"
-  svntest.actions.run_and_verify_svn("Error about nonexistent URL expected",
-                                     None, svntest.verify.AnyOutput,
+  svntest.actions.run_and_verify_svn(None, svntest.verify.AnyOutput,
                                      'export', nonexistent_url, export_target)
 
 def export_working_copy(sbox):
@@ -189,8 +188,7 @@ def export_over_existing_dir(sbox):
   # the export operation to fail.
   os.mkdir(export_target)
 
-  svntest.actions.run_and_verify_svn("No error where one is expected",
-                                     None, svntest.verify.AnyOutput,
+  svntest.actions.run_and_verify_svn(None, svntest.verify.AnyOutput,
                                      'export', sbox.wc_dir, export_target)
 
   # As an extra precaution, make sure export_target doesn't have
@@ -470,8 +468,7 @@ def export_nonexistent_file(sbox):
 
   export_target = sbox.add_wc_path('export')
 
-  svntest.actions.run_and_verify_svn("No error where one is expected",
-                                     None, svntest.verify.AnyOutput,
+  svntest.actions.run_and_verify_svn(None, svntest.verify.AnyOutput,
                                      'export', kappa_path, export_target)
 
 def export_unversioned_file(sbox):
@@ -485,8 +482,7 @@ def export_unversioned_file(sbox):
 
   export_target = sbox.add_wc_path('export')
 
-  svntest.actions.run_and_verify_svn("No error where one is expected",
-                                     None, svntest.verify.AnyOutput,
+  svntest.actions.run_and_verify_svn(None, svntest.verify.AnyOutput,
                                      'export', kappa_path, export_target)
 
 def export_with_state_deleted(sbox):
@@ -497,15 +493,14 @@ def export_with_state_deleted(sbox):
 
   # state deleted=true caused export to crash
   alpha_path = os.path.join(wc_dir, 'A', 'B', 'E', 'alpha')
-  svntest.actions.run_and_verify_svn(None, None, [], 'rm', alpha_path)
+  svntest.actions.run_and_verify_svn(None, [], 'rm', alpha_path)
   expected_output = svntest.wc.State(wc_dir, {
     'A/B/E/alpha' : Item(verb='Deleting'),
     })
   expected_status = svntest.actions.get_virginal_state(wc_dir, 1)
   expected_status.remove('A/B/E/alpha')
   svntest.actions.run_and_verify_commit(wc_dir,
-                                        expected_output, expected_status,
-                                        None, wc_dir)
+                                        expected_output, expected_status)
 
   export_target = sbox.add_wc_path('export')
   expected_output = svntest.wc.State(export_target, {
@@ -557,7 +552,7 @@ def export_HEADplus1_fails(sbox):
 
   sbox.build(create_wc = False, read_only = True)
 
-  svntest.actions.run_and_verify_svn(None, None, '.*No such revision.*',
+  svntest.actions.run_and_verify_svn(None, '.*No such revision.*',
                                      'export', sbox.repo_url, sbox.wc_dir,
                                      '-r', 38956)
 
@@ -613,7 +608,7 @@ def export_file_overwrite_fails(sbox):
 
   # Run it for source local
   open(os.path.join(tmpdir, 'iota'), 'w').write(not_iota_contents)
-  svntest.actions.run_and_verify_svn(None, [], '.*exist.*',
+  svntest.actions.run_and_verify_svn([], '.*exist.*',
                                      'export', iota_path, tmpdir)
 
   # Verify it failed
@@ -624,7 +619,7 @@ def export_file_overwrite_fails(sbox):
 
   # Run it for source URL
   open(os.path.join(tmpdir, 'iota'), 'w').write(not_iota_contents)
-  svntest.actions.run_and_verify_svn(None, [], '.*exist.*',
+  svntest.actions.run_and_verify_svn([], '.*exist.*',
                                      'export', iota_url, tmpdir)
 
   # Verify it failed
@@ -736,11 +731,11 @@ def export_with_url_unsafe_characters(sbox):
   # Create the file with special name and commit it.
   svntest.main.file_write(url_unsafe_path, 'This is URL unsafe path file.')
   svntest.main.run_svn(None, 'add', url_unsafe_path + '@')
-  svntest.actions.run_and_verify_svn(None, None, [], 'ci', '-m', 'log msg',
+  svntest.actions.run_and_verify_svn(None, [], 'ci', '-m', 'log msg',
                                      '--quiet', wc_dir)
 
   # Export the file and verify it.
-  svntest.actions.run_and_verify_svn(None, None, [], 'export',
+  svntest.actions.run_and_verify_svn(None, [], 'export',
                                      url_unsafe_path_url, export_target + '@')
 
   if not os.path.exists(export_target):
@@ -906,14 +901,14 @@ def export_file_overwrite_with_force(sbox):
 
   # Run it for WC export
   open(os.path.join(tmpdir, 'iota'), 'w').write(not_iota_contents)
-  svntest.actions.run_and_verify_svn(None, svntest.verify.AnyOutput,
+  svntest.actions.run_and_verify_svn(svntest.verify.AnyOutput,
                                      [], 'export', '--force',
                                      iota_path, tmpdir)
   svntest.actions.verify_disk(tmpdir, expected_disk)
 
   # Run it for URL export
   open(os.path.join(tmpdir, 'iota'), 'w').write(not_iota_contents)
-  svntest.actions.run_and_verify_svn(None, svntest.verify.AnyOutput,
+  svntest.actions.run_and_verify_svn(svntest.verify.AnyOutput,
                                      [], 'export', '--force',
                                      iota_url, tmpdir)
   svntest.actions.verify_disk(tmpdir, expected_disk)
@@ -956,7 +951,7 @@ def export_custom_keywords(sbox):
   export_file = os.path.join(export_target, 'alpha')
   os.remove(export_file)
   expected_output = ['A    %s\n' % export_file, 'Export complete.\n']
-  svntest.actions.run_and_verify_svn(None, expected_output, [],
+  svntest.actions.run_and_verify_svn(expected_output, [],
                                      'export', '--force',
                                      sbox.repo_url + '/A/B/E/alpha',
                                      export_target)
@@ -964,6 +959,114 @@ def export_custom_keywords(sbox):
   if open(export_file).read() != ''.join(alpha_content):
     raise svntest.Failure("wrong keyword expansion")
 
+@Issue(4427)
+def export_file_external(sbox):
+  "export file external from WC and URL"
+  sbox.build()
+
+  wc_dir = sbox.wc_dir
+
+  # Set 'svn:externals' property in 'A/C' to 'A/B/E/alpha'(file external),
+  C_path = os.path.join(wc_dir, 'A', 'C')
+  externals_prop = "^/A/B/E/alpha exfile_alpha"
+
+  tmp_f = sbox.get_tempname('prop')
+  svntest.main.file_append(tmp_f, externals_prop)
+  svntest.main.run_svn(None, 'ps', '-F', tmp_f, 'svn:externals', C_path)
+  svntest.main.run_svn(None,'ci', '-m', 'log msg', '--quiet', C_path)
+
+  # Update the working copy to receive file external
+  svntest.main.run_svn(None, 'up', wc_dir)
+
+  # Update the expected disk tree to include the external.
+  expected_disk = svntest.main.greek_state.copy()
+  expected_disk.add({
+      'A/C/exfile_alpha'  : Item("This is the file 'alpha'.\n"),
+      })
+
+  # Export from URL
+  export_target = sbox.add_wc_path('export_url')
+  expected_output = svntest.main.greek_state.copy()
+  expected_output.add({
+      'A/C/exfile_alpha'  : Item("This is the file 'alpha'.\r"),
+      })
+  expected_output.wc_dir = export_target
+  expected_output.desc[''] = Item()
+  expected_output.tweak(contents=None, status='A ')
+  svntest.actions.run_and_verify_export(sbox.repo_url,
+                                        export_target,
+                                        expected_output,
+                                        expected_disk)
+
+  # Export from WC
+  export_target = sbox.add_wc_path('export_wc')
+  expected_output = svntest.main.greek_state.copy()
+  expected_output.add({
+      'A/C/exfile_alpha'  : Item("This is the file 'alpha'.\r"),
+      })
+  expected_output.wc_dir = export_target
+  expected_output.desc['A'] = Item()
+  expected_output.tweak(contents=None, status='A ')
+  svntest.actions.run_and_verify_export(wc_dir,
+                                        export_target,
+                                        expected_output,
+                                        expected_disk)
+
+@Issue(4427)
+def export_file_externals2(sbox):
+  "exporting file externals"
+
+  sbox.build()
+  sbox.simple_mkdir('DIR', 'DIR2')
+
+  sbox.simple_propset('svn:externals', '^/iota file', 'DIR')
+  sbox.simple_propset('svn:externals', '^/DIR TheDir', 'DIR2')
+  sbox.simple_commit()
+  sbox.simple_update()
+
+  tmp = sbox.add_wc_path('tmp')
+  os.mkdir(tmp)
+
+  expected_output = svntest.wc.State(tmp, {
+    'file'          : Item(status='A '),
+  })
+  expected_disk = svntest.wc.State('', {
+    'file': Item(contents="This is the file 'iota'.\n")
+  })
+  # Fails in 1.8.8 and r1575909.
+  # Direct export of file external was just skipped
+  svntest.actions.run_and_verify_export(sbox.ospath('DIR/file'),
+                                        tmp,
+                                        expected_output,
+                                        expected_disk)
+
+  expected_output = svntest.wc.State(tmp, {
+    'DIR/file'           : Item(status='A '),
+  })
+  expected_disk = svntest.wc.State('', {
+    'file': Item(contents="This is the file 'iota'.\n")
+  })
+  # Fails in 1.8.8 (doesn't export file), passes in r1575909
+  svntest.actions.run_and_verify_export(sbox.ospath('DIR'),
+                                        os.path.join(tmp, 'DIR'),
+                                        expected_output,
+                                        expected_disk)
+
+  expected_output = svntest.wc.State(tmp, {
+    'DIR2/TheDir/file' : Item(status='A '),
+  })
+  expected_disk = svntest.wc.State('', {
+    'TheDir'      : Item(),
+    'TheDir/file' : Item(contents="This is the file 'iota'.\n")
+  })
+  # Fails in 1.8.8 (doesn't export anything),
+  # Fails in r1575909 (exports file twice; once as file; once as external)
+  svntest.actions.run_and_verify_export(sbox.ospath('DIR2'),
+                                        os.path.join(tmp, 'DIR2'),
+                                        expected_output,
+                                        expected_disk)
+
+
 ########################################################################
 # Run the tests
 
@@ -998,6 +1101,8 @@ test_list = [ None,
               export_to_current_dir,
               export_file_overwrite_with_force,
               export_custom_keywords,
+              export_file_external,
+              export_file_externals2
              ]
 
 if __name__ == '__main__':
